package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.Signer;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.RSABlindingParameters;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import java.math.BigInteger;
import java.security.SecureRandom;

public class PSSSigner implements Signer {
   public static final byte TRAILER_IMPLICIT = -68;
   private Digest digest;
   private AsymmetricBlockCipher cipher;
   private SecureRandom random;
   private int hLen;
   private int sLen;
   private int emBits;
   private byte[] salt;
   private byte[] mDash;
   private byte[] block;
   private byte trailer;

   public PSSSigner(AsymmetricBlockCipher cipher, Digest digest, int sLen) {
      this(cipher, digest, sLen, (byte)-68);
   }

   public PSSSigner(AsymmetricBlockCipher cipher, Digest digest, int sLen, byte trailer) {
      PSSSigner var10000 = this;
      PSSSigner var10002 = this;
      PSSSigner var10004 = this;
      PSSSigner var10006 = this;
      super();
      this.cipher = cipher;
      this.digest = digest;
      int var5;
      this.hLen = var5 = digest.getDigestSize();
      var10006.sLen = sLen;
      var10004.salt = new byte[sLen];
      var10002.mDash = new byte[sLen + 8 + var5];
      var10000.trailer = trailer;
   }

   private void clearBlock(byte[] block) {
      for(int var2 = 0; var2 != block.length; ++var2) {
         block[var2] = 0;
      }

   }

   private void ItoOSP(int i, byte[] sp) {
      sp[0] = (byte)(i >>> 24);
      sp[1] = (byte)(i >>> 16);
      sp[2] = (byte)(i >>> 8);
      sp[3] = (byte)(i >>> 0);
   }

   private byte[] maskGeneratorFunction1(byte[] Z, int zOff, int zLen, int length) {
      byte[] var5 = new byte[length];
      byte[] var6 = new byte[this.hLen];
      byte var7 = 4;
      byte[] var8 = new byte[4];
      int var9 = 0;
      this.digest.reset();

      int var10;
      while(var9 < length / (var10 = this.hLen)) {
         this.ItoOSP(var9, var8);
         this.digest.update(Z, zOff, zLen);
         this.digest.update(var8, 0, var7);
         this.digest.doFinal(var6, 0);
         int var11 = var9 * (var10 = this.hLen);
         System.arraycopy(var6, 0, var5, var11, var10);
         ++var9;
      }

      if (var9 * var10 < length) {
         this.ItoOSP(var9, var8);
         this.digest.update(Z, zOff, zLen);
         this.digest.update(var8, 0, var7);
         this.digest.doFinal(var6, 0);
         int var12;
         int Z1 = var9 * (var12 = this.hLen);
         var12 = length - var9 * var12;
         System.arraycopy(var6, 0, var5, Z1, var12);
      }

      return var5;
   }

   public void init(boolean forSigning, CipherParameters param) {
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom var10001 = (ParametersWithRandom)param;
         param = ((ParametersWithRandom)param).getParameters();
         this.random = var10001.getRandom();
      } else if (forSigning) {
         SecureRandom var3;
         var3 = new SecureRandom.<init>();
         this.random = var3;
      }

      this.cipher.init(forSigning, param);
      BigInteger forSigning1 = (param instanceof RSABlindingParameters ? ((RSABlindingParameters)param).getPublicKey() : (RSAKeyParameters)param).getModulus();
      this.block = new byte[((this.emBits = forSigning1.bitLength() - 1) + 7) / 8];
      this.reset();
   }

   public void update(byte b) {
      this.digest.update(b);
   }

   public void update(byte[] in, int off, int len) {
      this.digest.update(in, off, len);
   }

   public void reset() {
      this.digest.reset();
   }

   public byte[] generateSignature() {
      int var1;
      int var2;
      if (this.emBits < (var1 = this.hLen) * 8 + (var2 = this.sLen) * 8 + 9) {
         throw new DataLengthException("encoding error");
      } else {
         byte[] var10002 = this.mDash;
         this.digest.doFinal(var10002, var10002.length - var1 - var2);
         int var3;
         byte[] var6;
         if (this.sLen != 0) {
            this.random.nextBytes(this.salt);
            var3 = (var6 = this.mDash).length - (var2 = this.sLen);
            System.arraycopy(this.salt, 0, var6, var3, var2);
         }

         byte[] var7 = new byte[var1 = this.hLen];
         byte[] var10004 = this.mDash;
         this.digest.update(var10004, 0, var10004.length);
         this.digest.doFinal(var7, 0);
         byte[] var8 = this.block;
         int var4;
         int var5;
         var8[var8.length - (var4 = this.sLen) - 1 - (var5 = this.hLen) - 1] = 1;
         var5 = var8.length - var4 - var5 - 1;
         System.arraycopy(this.salt, 0, var8, var5, var4);
         var3 = this.block.length - this.hLen - 1;
         var6 = this.maskGeneratorFunction1(var7, 0, var1, var3);

         byte[] var10000;
         for(var3 = 0; var3 != var6.length; ++var3) {
            var10000 = this.block;
            var10000[var3] ^= var6[var3];
         }

         byte[] var10003 = var6 = this.block;
         var10003[0] = (byte)(var10003[0] & 255 >> var6.length * 8 - this.emBits);
         var3 = var10003.length - (var2 = this.hLen) - 1;
         System.arraycopy(var7, 0, var6, var3, var2);
         byte[] var10001 = var6 = this.block;
         var2 = var6.length - 1;
         var10001[var2] = this.trailer;
         var10000 = this.cipher.processBlock(var6, 0, var6.length);
         this.clearBlock(this.block);
         return var10000;
      }
   }

   public boolean verifySignature(byte[] signature) {
      int var2;
      int var3;
      if (this.emBits < (var2 = this.hLen) * 8 + (var3 = this.sLen) * 8 + 9) {
         return false;
      } else {
         PSSSigner var10001 = this;
         byte[] var10003 = this.mDash;
         this.digest.doFinal(var10003, var10003.length - var2 - var3);

         boolean var14;
         AsymmetricBlockCipher var15;
         try {
            var15 = var10001.cipher;
         } catch (Exception var10) {
            var14 = false;
            return false;
         }

         byte[] var10002 = signature;
         var10003 = signature;
         int signature1 = 0;

         byte[] var16;
         try {
            var16 = var15.processBlock(var10002, signature1, var10003.length);
         } catch (Exception var9) {
            var14 = false;
            return false;
         }

         var10002 = var16;
         var10003 = var16;
         PSSSigner var10004 = this;
         signature1 = 0;

         byte[] var19;
         try {
            var19 = var10004.block;
         } catch (Exception var8) {
            var14 = false;
            return false;
         }

         byte[] var12 = var19;

         int var18;
         int var20;
         try {
            var18 = var19.length;
            var20 = var10003.length;
         } catch (Exception var7) {
            var14 = false;
            return false;
         }

         var3 = var18 - var20;

         int var17;
         try {
            var17 = var10002.length;
         } catch (Exception var6) {
            var14 = false;
            return false;
         }

         int var4 = var17;

         try {
            System.arraycopy(var16, signature1, var12, var3, var4);
         } catch (Exception var5) {
            var14 = false;
            return false;
         }

         byte[] var10000 = signature = this.block;
         if (var10000[var10000.length - 1] != this.trailer) {
            this.clearBlock(signature);
            return false;
         } else {
            var2 = signature.length - (signature1 = this.hLen) - 1;
            var3 = signature.length - signature1 - 1;
            signature = this.maskGeneratorFunction1(signature, var2, signature1, var3);

            for(var2 = 0; var2 != signature.length; ++var2) {
               var10000 = this.block;
               var10000[var2] ^= signature[var2];
            }

            var10000 = signature = this.block;
            var10000[0] = (byte)(var10000[0] & 255 >> signature.length * 8 - this.emBits);

            for(signature1 = 0; signature1 != (var12 = this.block).length - (var3 = this.hLen) - (var4 = this.sLen) - 2; ++signature1) {
               if (var12[signature1] != 0) {
                  this.clearBlock(var12);
                  return false;
               }
            }

            if (var12[var12.length - var3 - var4 - 2] != 1) {
               this.clearBlock(var12);
               return false;
            } else {
               signature1 = var12.length - var4 - var3 - 1;
               byte[] var10006 = this.mDash;
               System.arraycopy(var12, signature1, var10006, var10006.length - var4, var4);
               var19 = this.mDash;
               this.digest.update(var19, 0, var19.length);
               var10003 = this.mDash;
               this.digest.doFinal(var10003, var10003.length - this.hLen);
               var2 = this.block.length - (signature1 = this.hLen) - 1;

               byte[] var13;
               for(signature1 = this.mDash.length - signature1; signature1 != (var13 = this.mDash).length; ++signature1) {
                  if ((this.block[var2] ^ var13[signature1]) != 0) {
                     this.clearBlock(var13);
                     this.clearBlock(this.block);
                     return false;
                  }

                  ++var2;
               }

               this.clearBlock(var13);
               this.clearBlock(this.block);
               return true;
            }
         }
      }
   }
}
